--*******************************************************************************************************************************
--
-- Description: SigFPGA top level test bench.
--				
--					
-- Rev History: 08-30-2014: Added a flag to signal the end of the simulation. Changed the TX data capture functionality to be controlled by target write.
--				08-01-2014: Changed input on the ADC ports to a synthetic sine wave.
--				03-31-2014:	Initial release.
--                                                  
--*******************************************************************************************************************************
                                             
library ieee,unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use unisim.VCOMPONENTS.all;
use work.model_specific_package.all;
use work.transceiver_package.all;
use work.tb_package.all;                      

entity SigFPGA_tb is
port (
    sim_status			: out std_logic
);                                                                                            
end SigFPGA_tb;

architecture structural of SigFPGA_tb is
    -- Tranciever simulation entity signals
	-- PCI Express bus signals produced by the Xilinx Endpoint Block Plus for PCI Express core generator.
	signal sys_rst_n		: std_logic;					
    signal pci_sys_clk		: std_logic;                                                      
    signal pci_exp_rxp		: std_logic_vector (PCI_LANES-1 downto 0);
    signal pci_exp_rxn		: std_logic_vector (PCI_LANES-1 downto 0);
    signal pci_exp_txp		: std_logic_vector (PCI_LANES-1 downto 0);
    signal pci_exp_txn		: std_logic_vector (PCI_LANES-1 downto 0);

	-- Clocks generated external to the FPGA.
    signal clk200_in		: std_logic;
    signal sample_clk_in  	: std_logic;
    signal sample_clk	  	: std_logic;

	-- RX data, over-range (ovr) and forwarded clock signals.
    signal rx_data_in  		: RX_BUS_IN;
    signal rx_ovr_in		: RX_1;
    signal rx_clk_in		: RX_1;
    signal tx_data_out  	: TX_BUS_OUT;
    signal tx_clk_in		: TX_1;
    signal tx_clk_out		: TX_1;       

	-- QDR II+ SRAM interface signals.
	signal qdriip_sys_clk_p	: std_logic;								-- System clock to both QDR II+ SRAM controllers.
	signal qdriip_sys_clk_n	: std_logic;								-- Differential pair (see above).
	signal qdriip_cq		: DUAL_1;									-- QDR II+ SRAM echo clock.
	signal qdriip_q			: DUAL_18;									-- QDR II+ SRAM synchronous data outputs from the SRAM.
	signal qdriip_qvld		: DUAL_1;									-- QDR II+ SRAM read data valid flag.
	signal qdriip_k_p		: DUAL_1;									-- QDR II+ SRAM input clock.
	signal qdriip_k_n		: DUAL_1;									-- QDR II+ SRAM differential pair (see above).
	signal qdriip_d			: DUAL_18;									-- QDR II+ SRAM synchronous data inputs to the SRAM.
	signal qdriip_sa		: DUAL_QDR;									-- QDR II+ SRAM synchronous address inputs to the SRAM.
	signal qdriip_w_n		: DUAL_1;									-- QDR II+ SRAM synchronous active low write enable.
	signal qdriip_r_n		: DUAL_1;									-- QDR II+ SRAM synchronous active low read enable.
	signal qdriip_bw_n		: DUAL_2;									-- QDR II+ SRAM synchronous byte enable inputs to the SRAM.
	signal qdriip_dll_off_n	: DUAL_1;									-- QDR II+ SRAM active low SRAM DLL disable.

	-- Inter-integrated circuit (I2C) bus signals.	
	signal i2c_scl 			: std_logic;
	signal i2c_sda_in 		: std_logic;
	signal i2c_sda_out 		: std_logic;
	signal i2c_direction 	: std_logic;

	-- Serial peripheral interface (SPI) bus signals.
    signal spi_clk			: std_logic;
    signal spi_sdi			: std_logic;
    signal spi_sdo			: std_logic;
    signal spi_direction	: std_logic;
    signal spi_csb_cmd     	: std_logic_vector(2 downto 0);

	-- Frequency synthesizer command interface signals, applies only to the programmable synthesizer option.
    signal synth_cmd		: std_logic_vector(2 downto 0);

	-- Hardware trigger inputs.
    signal coax_trig		: std_logic;
    signal gpio_trig		: std_logic;

	-- Alert notification from the power monitor and command/status port expander.
	signal alert_n			: std_logic;								-- Active low status alert.
	signal fan_clear		: std_logic;
	signal fan_status		: std_logic;
	
	-- Uncommitted bidirectional IO pins (GPIO connector and user connector).
	signal gpio_in			: std_logic_vector(GPIO_COUNT downto 1);		-- GPIO data input pins.
	signal gpio_out			: std_logic_vector(GPIO_COUNT downto 1);	-- GPIO data output pins.
	signal gpio_direction	: std_logic_vector(GPIO_COUNT downto 1);	-- GPIO tristate direction control.
	signal gpio_tristate	: std_logic;								-- Tristate all GPIO voltage translators.
	signal user_in			: std_logic_vector(USER_COUNT downto 1);		-- User data input pins.
	signal user_out			: std_logic_vector(USER_COUNT downto 1);	-- User data output pins.
	signal user_direction	: std_logic_vector(USER_COUNT downto 1);	-- User tristate direction control.
	signal user_tristate	: std_logic;								-- User all GPIO voltage translators.
	signal gpio_user_cmd	: std_logic_vector(2 downto 0);				-- GPIO/user voltage translator command interface (Bit 0: Clock, Bit 1: Data, Bit 2: Load).
	
	-- BPI configuration flash interface bus and controls.
	signal bpi_a			: std_logic_vector(22 downto 0);			-- Address bus.
	signal bpi_rs			: std_logic_vector(1 downto 0);				-- Revision address.
	signal bpi_rs_disable	: std_logic; 								-- Revision select disable.
	signal bpi_d_in			: std_logic_vector(15 downto 0);			-- Data input (read) bus.
	signal bpi_d_out		: std_logic_vector(15 downto 0);			-- Data output (write) bus.
	signal bpi_d_tristate	: std_logic;								-- Active high data bus tristate.
	signal bpi_adv_b		: std_logic; 								-- Active low address valid.
	signal bpi_fcs_b		: std_logic; 								-- Active low chip enable.
	signal bpi_fwe_b		: std_logic; 								-- Active low write enable.
	signal bpi_foe_b		: std_logic; 								-- Active low output enable.
	signal bpi_vpp			: std_logic; 								-- Active high enable voltage for program and erase operations.	

    -- Input data signals	     
    signal rst				: std_logic;								-- Global reset.
    signal adc_data			: RX_BUS_IN;								-- Data bus with synthetic ADC data.
    signal wave_start		: std_logic;								-- Control signal to generate synthetic ADC data.
    signal led_on			: std_logic_vector(1 downto 0);				-- Active high led control bits.
    
    -- TX data collection signals   
	signal txdata			: TX_BUS;									-- Data bus loaded with DAC data samples. 

begin

    XCVR : entity work.transceiver
	port map (                          
		sys_rst_n		=> sys_rst_n,
		pci_sys_clk		=> pci_sys_clk,
		pci_exp_rxp		=> pci_exp_rxp,
		pci_exp_rxn		=> pci_exp_rxn,
		pci_exp_txp		=> pci_exp_txp,
		pci_exp_txn		=> pci_exp_txn,
		clk200_in		=> clk200_in,
		sample_clk_in	=> sample_clk_in,
		rx_data_in 		=> adc_data,
		rx_ovr_in 		=> rx_ovr_in,
		rx_clk_in 		=> rx_clk_in,
		tx_data_out 	=> tx_data_out,
		tx_clk_in 		=> tx_clk_in,
		tx_clk_out		=> tx_clk_out,
		qdriip_sys_clk_p=> qdriip_sys_clk_p,
		qdriip_sys_clk_n=> qdriip_sys_clk_n,
		qdriip_cq		=> qdriip_cq,		
		qdriip_q		=> qdriip_q,		
		qdriip_qvld		=> qdriip_qvld,		
		qdriip_k_p		=> qdriip_k_p,		
		qdriip_k_n		=> qdriip_k_n,		
		qdriip_d		=> qdriip_d,	
		qdriip_sa		=> qdriip_sa,		
		qdriip_w_n		=> qdriip_w_n,		
		qdriip_r_n		=> qdriip_r_n,		
		qdriip_bw_n		=> qdriip_bw_n,		
		qdriip_dll_off_n=> qdriip_dll_off_n,
		i2c_scl 		=> i2c_scl,
		i2c_sda_in 		=> i2c_sda_in,
		i2c_sda_out 	=> i2c_sda_out,
		i2c_direction 	=> i2c_direction,
		spi_clk 		=> spi_clk,
		spi_sdi 		=> spi_sdi,
		spi_sdo 		=> spi_sdo,
		spi_direction	=> spi_direction,            
		spi_csb_cmd 	=> spi_csb_cmd,
		synth_cmd		=> synth_cmd,            
		led_on 			=> led_on,
		coax_trig		=> coax_trig,
		gpio_trig		=> gpio_trig,
		alert_n			=> alert_n,
		fan_clear		=> fan_clear,
		fan_status		=> fan_status,
		gpio_in			=> gpio_in,
		gpio_out		=> gpio_out,
		gpio_direction	=> gpio_direction,
		gpio_tristate	=> gpio_tristate,
		user_in			=> user_in,
		user_out		=> user_out,
		user_direction	=> user_direction,
		user_tristate	=> user_tristate,
		gpio_user_cmd	=> gpio_user_cmd,
		bpi_a			=> bpi_a,			
		bpi_rs			=> bpi_rs,			
		bpi_rs_disable	=> bpi_rs_disable,	
		bpi_d_in		=> bpi_d_in,		
		bpi_d_out		=> bpi_d_out,		
		bpi_d_tristate	=> bpi_d_tristate,	
		bpi_adv_b		=> bpi_adv_b,		
		bpi_fcs_b		=> bpi_fcs_b,		
		bpi_fwe_b		=> bpi_fwe_b,		
		bpi_foe_b		=> bpi_foe_b,		
		bpi_vpp			=> bpi_vpp				

	);

   	rst <= not(sys_rst_n);
	 	          
	-------------------------------------------
	-- Generate RX Data 
	-------------------------------------------
	
	RX_DATA_INPUT : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate  
		SYNTH_SINE_WAVE : entity work.rx_synthetic_sin_wave 
		port map(
    		clk				=> rx_clk_in(i),
    		rst				=> rst,
    		wave_start		=> '1',
    		synth_wave		=> adc_data(i)  
    		);
   	end generate;
   	
	-------------------------------------------
	-- TX Data Collection
 	-------------------------------------------
	
	TX_SHARE_CH : if TX_DATA_SHARE = 2 generate
		TX_NUM_SHARE_CH : for i in zero_to_one(integer(TX_CHANNEL_COUNT/TX_DATA_SHARE))+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1 generate
			TX_DATA : process (tx_clk_out(i), rst) 
			begin				                                                                                                 
				if rst = '1' then 
					txdata(i) <= (others => '0');
					
				elsif rising_edge(tx_clk_out(i)) then  
					txdata(i) <= tx_data_out(i);
						                            
					
				elsif falling_edge(tx_clk_out(i)) then   				
					txdata(i+1) <= tx_data_out(i);
					
				end if;
			end process;
		end generate;
	end generate;
	  	
	-------------------------------------------   
	-- Clock Generator
	-------------------------------------------
             
	CLK200_GEN : process
	begin	
        clk200_in <= '1';
        wait for CLK200_PD/2.0;                         
        loop                                          
            clk200_in <= '0', '1' after CLK200_PD/2.0;
            wait for CLK200_PD;                                                            
        end loop;         
	end process;
 
	SAMPLE_CLK_GEN : process
	begin	
        sample_clk <= '1';
        wait for SAMPLE_CLK_PD/2.0;                         
        loop                                          
            sample_clk <= '0', '1' after SAMPLE_CLK_PD/2.0;
            wait for SAMPLE_CLK_PD;                                                            
        end loop;         
	end process;
	
	RX : if RX_CHANNEL_COUNT > 0 generate 
		RX_GEN : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			RX_CLK_GEN : process
			begin	
				rx_clk_in(i) <= '1';
				wait for RX_CLK_PD/2.0;                         
				loop                                          
					rx_clk_in(i) <= '0', '1' after RX_CLK_PD/2.0;
					wait for RX_CLK_PD;                                                            
				end loop;         
			end process;
		end generate;
	end generate;     
	
	-- Assign unused signals if there are no RX channels.
	NO_RX : if RX_CHANNEL_COUNT = 0 generate
		rx_clk_in(RX_CHANNEL_COUNT+1) <= '0';
	end generate;
	
	TX : if TX_CHANNEL_COUNT > 0 generate
		TX_GEN : for i in RX_CHANNEL_COUNT+1 to zero_to_one(TX_CHANNEL_COUNT)+RX_CHANNEL_COUNT generate
			TX_CLK_GEN : process
			begin	                         
	    	    tx_clk_in(i) <= '1';
	    	    wait for TX_CLK_PD/2.0;                         
	    	    loop                                          
	    	        tx_clk_in(i) <= '0', '1' after TX_CLK_PD/2.0;
	    	        wait for TX_CLK_PD;                                                            
	    	    end loop;                      
			end process;                               
	 	end generate;                
 	end generate;
  
 	-- Assign unused signals if there are no TX channels.
	NO_TX : if TX_CHANNEL_COUNT = 0 generate
		tx_clk_in(RX_CHANNEL_COUNT+1) <= '0';
	end generate;

	SYS_CLK_GEN : process
	begin	
        pci_sys_clk <= '1';
        wait for SYS_CLK_PD/2.0;                         
        loop                                          
            pci_sys_clk <= '0', '1' after SYS_CLK_PD / 2.0;
            wait for SYS_CLK_PD;                                                            
        end loop;                                            
	end process;
	                                
	--Generate system reset
	SYS_RST_GEN : process
	begin
		sys_rst_n <= '0';
		wait for 650 ns;                    
		sys_rst_n <= '1';
		wait;
	end process;    
	
  	--	The RX sample clock is too fast for the FPGA, for the Model 376 use the divided RX1 data clock instead.
  	sample_clk_in <= rx_clk_in(1) when MODEL = x"376" else sample_clk; 
	sim_status <= led_on(0);
end structural;
