--*******************************************************************************************************************************
--
-- Description: This module performs all PCI Express write/read operations to registers not associated with a specific channel.
--				The AXI_WIDTH constant determines whether slave_dataout is 64-bit or 128-bit aligned.
--					
-- Rev History: 10-20-2014: Changed the register clear operation on the PCI performance monitor.
--				08-20-2014: Added QDR address width to the reported model specific features and updated PCI performance monitor.
--				05-28-2014: Added PCI performance monitor.
--				05-21-2014:	Consolidated QDR II+ SRAM registers and added control signal for the core calibration function.
-- 				09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity bar2_common is
port (                                                                                                                              
    -- Reset and clock.                                                                                      
	rst    				: in std_logic;									-- Active high asynchronous reset.
	clk		        	: in std_logic;									-- Bus clock.
	
    -- PCI Express write transaction interface.
	slv_addr 			: in std_logic_vector(15 downto 2);				-- Refer to the PLDA EZDMA2 reference manual.
    slv_dataout     	: in std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	bytevalid 		: in std_logic_vector(1 downto 0);					-- Logic one indicates that all PCI byte valids are active.
	bar2_write 			: in std_logic;									-- Logic one indicates that the current PCI write request is directed at BAR2.
                                                                                                                              
    -- PCI Express read transaction interface.
	bar2_slave_read		: in std_logic;									-- Logic one indicates that the current PCI read request is directed at BAR2.
	bar2_rddata_common	: out std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data return from read operation on BAR2 common register.
    dma_rdaddr      	: in std_logic_vector(15 downto 3);				-- Refer to the PLDA EZDMA2 reference manual.

	-- QDR II+ SRAM control and status signals.
	qdr_wr_data			: out std_logic_vector(71 downto 0);			-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	qdr_bw_n			: out std_logic_vector(7 downto 0);				-- Eight byte enables assigned to the QDR II+ SRAM write data values.
	qdr_rd_data			: in std_logic_vector(71 downto 0);				-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	qdr_addr			: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for current operation.
	qdr_cmd				: out DUAL_2;									-- Single period write (0) and read (1) command strobes in the PCI clock domain.
	qdr_rd_valid		: in std_logic;									-- Logic one indicates that read data is available from the QDR II+ SRAM.
 	qdr_calib			: out std_logic;								-- Logic one initiates calibration of the QDR II+ SRAM.

	-- PCI performance monitor.
 	pci_measurement		: out std_logic_vector(9 downto 0);				-- Control for PCI throughput and latency measurments.
	pci_throughput		: in std_logic_vector(63 downto 0);				-- Report the measured PCI DMA throughput in bytes/sec (transmitter=63:32, receiver=31:0).
	pci_latency			: in std_logic_vector(63 downto 0);				-- Report the maximum PCI DMA latency in 4 ns clock periods (transmitter=63:32, receiver=31:0).

	-- Command, status and error signals.
	soft_trig			: out std_logic;								-- Software trigger requested over the PCIe bus.
	led_control 		: out std_logic_vector(1 downto 0);  			-- Logic one illuminates each of the two LEDs.
	address_error 		: out std_logic_vector(1 downto 0)				-- Logic one indicates an invalid address for this decoder.
);                                                                                                                     	
end bar2_common;                                                                                                      	   
                                                                                                                               	 
architecture structural of bar2_common is
                                                                                                                               	
	signal led_control_reg	: std_logic_vector(1 downto 0);				-- Mask bits to disable specific sources from illuminating the LEDs.
	signal qdr_bw_n_reg		: std_logic_vector(7 downto 0);				-- Eight active low byte enables assigned to the QDR II+ SRAM write data values.
	signal qdr_addr_reg		: std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for current operation.

begin
	
--***************************************************** WRITE TRANSACTIONS *****************************************************

   process (clk,rst)
    begin
        if rst='1' then
			address_error(0) <= '0';
			soft_trig <= '0';
        	led_control_reg <= (others => '0');
			for i in 0 to 1 loop
        		qdr_wr_data <= (others => '0');
        		qdr_bw_n_reg <= (others => '0');
        		qdr_addr_reg <= (others => '0');
				qdr_cmd(i) <= (others => '0');
			end loop;
            pci_measurement <= (others => '0');

        elsif rising_edge(clk) then

            -- Target write decoder that loads slave data into the selected register for a target write completion.
            if bar2_write = '1' and slv_addr(15 downto 12) = x"0" then
                case slv_addr(15 downto 3) & "000" is
                
                	-- x"0000" is a read only register.
					
                    -- Generate a single period software trigger when there is a write to address x"0010".
                    when x"0010" =>
                        if bytevalid(0) = '1' then
							-- Issue a single period enable strobe.
							soft_trig <= '1';
                        end if;
						
					-- This is a 2-bit register used to control LED illumination.
                    when x"0020" =>
						if bytevalid(0) = '1' then led_control_reg <= slv_dataout(1 downto 0); end if;
						
					-- Two lower 18-bit QDR II+ SRAM write data words.
                    when x"0030" =>
						if bytevalid(1) = '1' then qdr_wr_data(35 downto 18) <= slv_dataout(49 downto 32); end if;
						if bytevalid(0) = '1' then qdr_wr_data(17 downto 0) <= slv_dataout(17 downto 0); end if;
						
					-- Two upper 18-bit QDR II+ SRAM write data words.
                    when x"0038" =>
						if bytevalid(1) = '1' then qdr_wr_data(71 downto 54) <= slv_dataout((AXI_WIDTH-64)+49 downto (AXI_WIDTH-64)+32); end if;
						if bytevalid(0) = '1' then qdr_wr_data(53 downto 36) <= slv_dataout((AXI_WIDTH-64)+17 downto (AXI_WIDTH-64)); end if;
						
					-- Address and byte enables for QDR II+ SRAM.
                    when x"0040" =>
						if bytevalid(1) = '1' then qdr_bw_n_reg <= slv_dataout(39 downto 32); end if;
						if bytevalid(0) = '1' then qdr_addr_reg <= slv_dataout(QDR_ADDR_WIDTH-1 downto 0); end if;
						
					-- Read/write/calibration command strobes to QDR II+ SRAM-A (0) and SRAM-B (1).
					-- Decoder:  x0 = SRAM-A Write, x1 = SRAM-A Read, x2 = SRAM-B Write, x3 = SRAM-B Read, x4 = SRAM Calibration
                    when x"0048" =>
						if bytevalid(0) = '1' then
							if slv_dataout(AXI_WIDTH-62) = '1' then
								qdr_calib <= '1';
							elsif slv_dataout(AXI_WIDTH-64) = '0' then
								-- Issue a one clock period write command.
								qdr_cmd(0)(0) <= not(slv_dataout(AXI_WIDTH-63));
								qdr_cmd(1)(0) <= slv_dataout(AXI_WIDTH-63);
							else
								-- Issue a one clock period read command.
								qdr_cmd(0)(1) <= not(slv_dataout(AXI_WIDTH-63));
								qdr_cmd(1)(1) <= slv_dataout(AXI_WIDTH-63);
							end if;
						end if;
												
					-- PCI throughput monitor.
                    when x"0050" =>
						if bytevalid(0) = '1' then
							pci_measurement(7 downto 0) <= slv_dataout(7 downto 0);
							pci_measurement(8) <= slv_dataout(8);
						end if;
						
					-- PCI latency monitor.
                    when x"0058" =>
						if bytevalid(0) = '1' then 
							-- Issue a single period clear strobe.
							pci_measurement(9) <= '1';
						end if;
						
					-- Return an error if the address is not valid.
                    when others  =>
						address_error(0) <= '1';
						
                end case;    
			else
				address_error(0) <= '0';
				soft_trig <= '0';
				qdr_cmd(0) <= (others => '0');
				qdr_cmd(1) <= (others => '0');
				qdr_calib <= '0';
				pci_measurement(9 downto 8) <= (others => '0');
           end if; 
            
        end if; 
    end process;
    
--***************************************************** READ TRANSACTIONS ******************************************************

    process (clk,rst)
    begin
        if rst = '1' then
            bar2_rddata_common <= (others => '0');
			address_error(1) <= '0';
        elsif rising_edge(clk) then
  
            -- Target read decoder that loads selected register contents onto the DMA bus for a target read completion.
            if bar2_slave_read = '1' and slv_addr(15 downto 12) = x"0" then
            
            	-- Set the default value to zero for cases that do not assign all of the bits.
				bar2_rddata_common <= (others => '0');
				
                case dma_rdaddr(15 downto 3) & "000" is
                
					-- Model specific feature identification.
					when x"0000" =>
						bar2_rddata_common(59 downto 52) <= std_logic_vector(to_unsigned(QDR_ADDR_WIDTH,8));
						bar2_rddata_common(51 downto 48) <= std_logic_vector(to_unsigned(TX_CHANNEL_COUNT,4));
						bar2_rddata_common(47 downto 44) <= std_logic_vector(to_unsigned(RX_CHANNEL_COUNT,4));
						bar2_rddata_common(43 downto 32) <= MODEL;
						bar2_rddata_common(31 downto 0) <= REVISION;
						
                	--  x"010" is a write only register.
						
					-- This is a 2-bit register used to control LED illumination.
                    when x"0020" =>
						bar2_rddata_common(1 downto 0) <= led_control_reg;
                    
					-- Two lower 18-bit QDR II+ SRAM read data words.
					when x"0030" =>
						bar2_rddata_common(49 downto 32) <= qdr_rd_data(35 downto 18);
						bar2_rddata_common(17 downto 0) <= qdr_rd_data(17 downto 0);
											
					-- Two upper 18-bit QDR II+ SRAM read data words.
					when x"0038" =>
						bar2_rddata_common((AXI_WIDTH-64)+49 downto (AXI_WIDTH-64)+32) <= qdr_rd_data(71 downto 54);
						bar2_rddata_common((AXI_WIDTH-64)+17 downto (AXI_WIDTH-64)) <= qdr_rd_data(53 downto 36);
										
					-- Address and byte enables for QDR II+ SRAM.
					when x"0040" =>
						bar2_rddata_common(39 downto 32) <= qdr_bw_n_reg;
						bar2_rddata_common(QDR_ADDR_WIDTH-1 downto 0) <= qdr_addr_reg;
										
					-- Read command status from QDR II+ SRAM.
					when x"0048" =>
						bar2_rddata_common(AXI_WIDTH-64) <= qdr_rd_valid;
																			
					-- PCI throughput monitor.
					when x"0050" =>
						bar2_rddata_common(63 downto 0) <= pci_throughput;
										
					-- PCI latency monitor.
					when x"0058" =>
						bar2_rddata_common((AXI_WIDTH-64)+63 downto (AXI_WIDTH-64)) <= pci_latency;
										
					-- Return an error if the address is not valid.
                    when others  =>                                                                                                 
						bar2_rddata_common(31 downto 0) <= x"badaddb2";
						address_error(1) <= '1';

	            end case;
			else
				address_error(1) <= '0';
            end if;
                        
        end if;
    end process;

--****************************************************** PORT ASSIGNMENTS ******************************************************

    led_control <= led_control_reg;
	qdr_addr <= qdr_addr_reg;
	qdr_bw_n <= qdr_bw_n_reg;
		
end structural;
