--*******************************************************************************************************************************
--
-- Description: This module sends data to a variety of D/A converter(DAC) chips.  The bus type supports multiple TX channels
--				and stuffing of multiple samples per clock cyle to accomodate DDR outputs.  When samples are stuffed, they are
--				chronologically ordered from LSB to MSB.
--					
-- Rev History: 09-04-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity dac_interface is 
	port (
    -- Reset and clock.                                                                                      
	rst         	: in std_logic;		-- Active high asynchronous reset.       
	tx_clk			: in std_logic;		-- Clock used to register DAC sample data.
	
	-- DAC data supplied from the application logic.
	tx_data_in		: in TX_BUS;		-- Data bus carrying DAC samples.
	
	-- The following signals connect to DAC devices through FPGA pins.
	tx_data_out		: out TX_BUS_OUT;	-- Data bus carrying DAC samples.
	tx_clk_out		: out TX_1			-- Internal single ended version of DAC forwarded clock.

	);	
	end dac_interface;	

architecture behavioral of dac_interface is

begin                    
 	
--************************************************** EXTERNAL PIN ASSIGNMENTS **************************************************

	-- Instantiate the requested number of transmitter channels.
	DAC_CHANNEL : for i in RX_CHANNEL_COUNT+1 to integer(TX_CHANNEL_COUNT/TX_DATA_SHARE)+RX_CHANNEL_COUNT generate
	
		DAC_DATA : for j in 0 to TX_ITEM_SIZE-1 generate
			-- Use double data rate (DDR) output registers in the IOB to transmit data on the rising and falling edge of the sample clock.
			-- Note: The DDR register is used on SDR outputs to ensure that the signal is registered inside the IOB.
			DATA_DDR : ODDR
				GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE")
				PORT MAP(Q => tx_data_out(i)(j), C => tx_clk, CE => '1', D1 => tx_data_in((2*i)-(RX_CHANNEL_COUNT+1))(j), D2 => tx_data_in((2*i)-(RX_CHANNEL_COUNT+1)+integer(TX_DATA_SHARE/2))(j), R => rst, S => '0');
		end generate;
	
		-- Instantiate a DDR clock forward if requested.
		CLK_FORWARD : if DAC_CLK_SOURCES > 0 generate
			CLK_FWD : if (i-(RX_CHANNEL_COUNT+1)) mod integer((TX_CHANNEL_COUNT/TX_DATA_SHARE)/zero_to_one(DAC_CLK_SOURCES)) = 0 generate
				-- Use a double data rate (DDR) register to generate the forwarded DAC clock.
				CLK_DDR : ODDR
					generic map (DDR_CLK_EDGE => "SAME_EDGE") 
					port map (D1 => '1', D2 => '0', C => tx_clk, CE => '1', Q => tx_clk_out(i), R => rst, S => '0');
			end generate;
		end generate;
		
	end generate;
	
end behavioral;
