--*******************************************************************************************************************************
--
-- Description: This module throttles data flow based on an external enable or a requested sample count.
--					
-- Rev History: 10-20-2014:	Preload the sample counter so that the threshold flag will be set one clock period earlier.
--				09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity rx_flow_control is
port (
    -- Clocks and synchronization.
	rst 			: in std_logic;								-- Active high asynchronous reset.
	rx_clk 			: in std_logic;								-- Sample clock coincident with the input sample data.
	enable 			: in std_logic;								-- Active high enable to allow data items to pass.
	
    -- Data busses carrying sample data.
	data_item		: in std_logic_vector((RX_ITEM_STUFF*RX_ITEM_SIZE)-1 downto 0);	-- Input data items that may be stuffed.
	payload_data	: out std_logic_vector(63 downto 0);		-- Output bus carrying data to perform a DMA write transaction.
	payload_valid	: out std_logic_vector((AXI_WIDTH/64)-1 downto 0);	-- Active high payload data valid flag.
	
    -- Configuration, command, and status.                                                                                      
	sample_count	: in std_logic_vector(31 downto 0);			-- Number of output samples to pass when this option is selected (below).
	stop_on_count	: in std_logic;								-- Logic one throttles the data after the requested number of output samples pass.
	count_done		: out std_logic								-- Logic one indicates that the requested number of output samples have passed.
);
end rx_flow_control;

architecture structural of rx_flow_control is

	signal item_16bit	 	: std_logic_vector((16*RX_ITEM_STUFF)-1 downto 0);	-- Stuffed data items sign extended to 16 bits.
	signal sample_counter 	: std_logic_vector(31 downto 0);	-- Count of the number of samples that have passed.
	signal item_counter 	: unsigned(1 downto 0);				-- Counter to stripe data items across 64-bit payloads.
	signal payload_select 	: std_logic;						-- Counter to stripe 64-bit payloads across the DMA FIFO width.
	signal threshold		: std_logic; 						-- Logic one indicates that the requested number of output samples have passed.
	signal load			: std_logic; 						-- Logic one loads the sample counter.
	
begin

--******************************************************** FLOW CONTROL ********************************************************

	process (rx_clk,rst)
    begin
		if rst='1' then
			payload_select <= '0';
			payload_valid <= (others => '0');
			item_counter <= (others => '0');

        elsif rising_edge(rx_clk) then
        				
			-- Data flow can be disabled by the synchronizer or the local sample counter.
			if enable = '0' or (threshold = '1' and stop_on_count = '1') then
				-- Set the payload select to the lower 64 bits of the DMA FIFO input.
				payload_select <= '0';
				-- Disable writes to all 64-bit DMA FIFO inputs.
				payload_valid <= (others => '0');
				-- Reset the item counter.
				item_counter <= (others => '0');
			else
				-- Wait for a complete 64-bit payload to be generated before sending it to the FIFO.
				if item_counter = to_unsigned((4/RX_ITEM_STUFF)-1,2) then
					-- There are two DMA FIFO inputs when AXI_WIDTH is set to 128 bits.
					if AXI_WIDTH/64 = 2 then
						-- Set one of the two payload valid bits.
						if payload_select = '0' then
							payload_valid(0) <= '1';
							payload_valid((AXI_WIDTH/64)-1) <= '0';
						else
							payload_valid(0) <= '0';
							payload_valid((AXI_WIDTH/64)-1) <= '1';
						end if;
						-- Toggle between the two 64-bit DMA FIFO inputs.
						payload_select <= not(payload_select);
					-- There is one DMA FIFO input when AXI_WIDTH is set to 64 bits.
					else
						-- Set the payload valid bit.
						payload_valid(0) <= '1';
						payload_select <= '0';
					end if;                                              
					-- Reset the item counter.
					item_counter <= (others => '0');
				else
					-- Disable writes to all 64-bit DMA FIFO inputs.
					payload_valid <= (others => '0');
					-- Increment the item counter.
					item_counter <= item_counter + "01";
				end if;
			end if;
        end if;
    end process;

--***************************************************** PAYLOAD FORMATTER ******************************************************

	-- Sign extend the data items to 16 bits for simplification of this demonstration.
	RESTUFF : for i in 0 to RX_ITEM_STUFF-1 generate
		MAKE_16BIT : for j in 0 to 15 generate
			ITEM_BITS : if j < RX_ITEM_SIZE generate
				item_16bit((16*i)+j) <= data_item((RX_ITEM_SIZE*i)+j);
			end generate;
			SIGN_BITS : if j >= RX_ITEM_SIZE generate
				item_16bit((16*i)+j) <= data_item((RX_ITEM_SIZE*(i+1))-1);
			end generate;
		end generate;
	end generate;
	
	-- Build the 64-bit payload based on the number of data items that are generated each sample clock.
	process (rx_clk,rst)
    begin
		if rst='1' then
			payload_data <= (others => '0');

        elsif rising_edge(rx_clk) then
			case item_counter is
				-- This count will always occur, regardless of the number of data items generated per sample clock.
				when "00" =>
					payload_data((16*RX_ITEM_STUFF)-1 downto 0) <= item_16bit((16*RX_ITEM_STUFF)-1 downto 0);
				-- This count will only occur if there is either one or two data items generated per sample clock.
				when "01" =>
					payload_data((32*(2-(RX_ITEM_STUFF mod 2)))-1 downto (16*(2-(RX_ITEM_STUFF mod 2)))) <= item_16bit((16*(2-(RX_ITEM_STUFF mod 2)))-1 downto 0);
				-- This count will only occur if there is one data item generated per sample clock.
				when "10" =>
					payload_data(47 downto 32) <= item_16bit(15 downto 0);
				-- This count will only occur if there is one data item generated per sample clock.
				when "11" =>
					payload_data(63 downto 48) <= item_16bit(15 downto 0);
				when others  =>
					null;
			end case;
        end if;
    end process;

--*************************************************** SAMPLE COUNT THROTTLE ****************************************************

	process (rx_clk,rst)
    begin
		if rst='1' then
			threshold <= '0';

        elsif rising_edge(rx_clk) then
        				
			-- Set the threshold flag when the requested number of samples have been collected.
			-- Note: There may be more than one data item collected per sample clock.
			if sample_counter(31-(RX_ITEM_STUFF/2) downto 0) = sample_count(31 downto RX_ITEM_STUFF/2) then
				threshold <= '1';
			elsif enable = '0' then
				threshold <= '0';
			end if;

        end if;
    end process;

  	-- Load the sample counter when the channel is disabled.
	load <= not(enable);
       
	-- Use a DSP48 block to implement the sample counter for performance.
	SAMPLE_CNT : entity work.counter_32
	port map (
		clk		=> rx_clk,
		sclr	=> '0',
		ce		=> stop_on_count,
		load	=> load,
		l		=> x"00000001",
		q		=> sample_counter
	);

	-- Map the threshold flag to an output port.
	count_done <= threshold;
			
end structural;
