--*******************************************************************************************************************************
--
-- Description: This module throttles data flow based on an external enable or a requested sample count.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity tx_flow_control is
port (
    -- Resets and clocks.
	rst 			: in std_logic;								-- Active high reset.
	clear 			: in std_logic;								-- Active high clear strobe.
	tx_clk 			: in std_logic;								-- Sample clock coincident with the input sample data.
	enable 			: in std_logic;								-- Active high enable to allow data items to pass.
	
    -- Data busses carrying sample data.
	payload_data		: in std_logic_vector(AXI_WIDTH-1 downto 0);	-- Input bus carrying data from a DMA read transaction.
	payload_data_req	: out std_logic_vector((AXI_WIDTH/64)-1 downto 0);	-- Active high payload request flag.
	payload_valid		: in std_logic;									-- Active high payload valid flag.
	data_item			: out std_logic_vector((TX_ITEM_STUFF*TX_ITEM_SIZE)-1 downto 0);	-- Output data items that may be stuffed.
	
    -- Configuration, command, and status.                                                                                      
	sample_count	: in std_logic_vector(31 downto 0);			-- Number of output samples to pass when this option is selected (below).
	stop_on_count	: in std_logic;								-- Logic one throttles the data after the requested number of output samples pass.
	count_done		: out std_logic;							-- Logic one indicates that the requested number of output samples have passed.
	
    -- Control signals to prime the datapath after it is cleared.                                                                                      
	prime_status	: out std_logic								-- Logic one indicates that this module is primed and output data is available.
);
end tx_flow_control;

architecture structural of tx_flow_control is

	signal item_16bit	 	: std_logic_vector((16*RX_ITEM_STUFF)-1 downto 0);	-- Stuffed data items sign extended to 16 bits.
	signal sample_counter 	: std_logic_vector(31 downto 0);	-- Count of the number of samples that have passed.
	signal item_counter 	:  unsigned(1 downto 0);			-- Counter to stripe data items across 64-bit payloads.
	signal payload_select 	: std_logic;						-- Counter to read 64-bit payloads striped across the DMA FIFO width.
	signal threshold		: std_logic; 						-- Logic one indicates that the requested number of output samples have passed.
	signal count_enable		: std_logic; 						-- Logic one enables the sample counter.
	signal prime_active		: std_logic;						-- Flag to consolidate all of the conditions that activate a data priming operation.
	signal prime_done		: std_logic;						-- Logic one indicates that this module is primed with valid data at the output.
	signal payload_active	: std_logic_vector(63 downto 0);	-- Active payload input from the FIFO.
	
begin

--******************************************************** FLOW CONTROL ********************************************************

	process(tx_clk,rst)
    begin
		if rst='1' then
		
			prime_done <= '0';
			payload_select <= '0';
			payload_data_req <= (others => '0');
			item_counter <= (others => '0');
			
        elsif rising_edge(tx_clk) then
        
			if clear = '1' then
				-- Reset the data prime status bit.
				prime_done <= '0';
				-- Set the payload select to the lower 64 bits of the DMA FIFO output.
				payload_select <= '0';
				-- Disable reads from all 64-bit DMA FIFO inputs.
				payload_data_req <= (others => '0');
				-- Reset the item counter.
				item_counter <= (others => '0');
			elsif enable = '1' or prime_active = '1' then
				-- Wait for a complete 64-bit payload to be stripped before requesting another from the FIFO.
				if item_counter = to_unsigned((4/TX_ITEM_STUFF)-1,2) then
					-- There are two DMA FIFO outputs when AXI_WIDTH is set to 128 bits.
					if AXI_WIDTH/64 = 2 then
						-- Set one of the two payload request bits.
						if payload_select = '0' then
							payload_data_req(0) <= '1';
							payload_data_req((AXI_WIDTH/64)-1) <= '0';
						else                                              
							payload_data_req(0) <= '0';
							payload_data_req((AXI_WIDTH/64)-1) <= '1';
						end if;
						-- Toggle between the two 64-bit DMA FIFO inputs.
						payload_select <= not(payload_select);
					-- There is one DMA FIFO input when AXI_WIDTH is set to 64 bits.
					else
						-- Set the payload request bit.
						payload_data_req(0) <= '1';
						payload_select <= '0';
					end if;
					-- Reset the item counter.
					item_counter <= "00";
				else
					-- Disable reads from all 64-bit DMA FIFO inputs.
					payload_data_req <= (others => '0');
					-- Increment the item counter.
					item_counter <= item_counter + "01";
				end if;
				
				-- This function is completely primed after one clock with valid input data.
				prime_done <= '1';
				
			end if;
        end if;
    end process;
				
	-- Generate the prime active flag based on current status.
	prime_active <= payload_valid and not(prime_done);
	-- Set the status bit if the upstream datapath is primed and there is data available to prime the downstream datapath.
	prime_status <= prime_done and payload_valid;
			
--***************************************************** PAYLOAD FORMATTER ******************************************************

	-- Strip the 64-bit payload based on the number of data items that are generated each sample clock.
	process (tx_clk,rst)
    begin
		if rst='1' then
			item_16bit <= (others => '0');

        elsif rising_edge(tx_clk) then
			if enable = '1' or prime_active = '1' then
				case item_counter is
				-- This count will always occur, regardless of the number of data items generated per sample clock.
				when "00" =>
					item_16bit((16*TX_ITEM_STUFF)-1 downto 0) <= payload_active((16*TX_ITEM_STUFF)-1 downto 0);
				-- This count will only occur if there is either one or two data items generated per sample clock.
				when "01" =>
					item_16bit((16*(2-(TX_ITEM_STUFF mod 2)))-1 downto 0) <= payload_active((32*(2-(TX_ITEM_STUFF mod 2)))-1 downto (16*(2-(TX_ITEM_STUFF mod 2))));
				-- This count will only occur if there is one data item generated per sample clock.
				when "10" =>
					item_16bit(15 downto 0) <= payload_active(47 downto 32);
				-- This count will only occur if there is one data item generated per sample clock.
				when "11" =>
					item_16bit(15 downto 0) <= payload_active(63 downto 48);
				when others  =>
					null;
				end case;
			end if;
        end if;
    end process;
				
	-- Convert the 16-bit payload data item size to the actual item size expected by the DAC.
	UNSTUFF : for i in 0 to TX_ITEM_STUFF-1 generate
		ITEM_BITS : for j in 0 to TX_ITEM_SIZE-1 generate
			-- Set the output data to zero when the channel is cleared or as soon as a channel is disabled.
			data_item((TX_ITEM_SIZE*i)+j) <= '0' when clear = '1' or enable = '0' or threshold = '1' else item_16bit((16*i)+j);
		end generate;
	end generate;
	
--*************************************************** SAMPLE COUNT THROTTLE ****************************************************

	process (tx_clk,rst)
    begin
		if rst='1' then
			threshold <= '0';

        elsif rising_edge(tx_clk) then
        				
			-- Set the threshold flag when the requested number of samples have been sent.
			-- Note: There may be more than one data item sent per sample clock.
			if stop_on_count = '1' and sample_counter(31-(TX_ITEM_STUFF/2) downto 0) = sample_count(31 downto TX_ITEM_STUFF/2) then
				threshold <= '1';
			elsif enable = '0' then
				threshold <= '0';
			end if;
			
        end if;
    end process;

    -- Select the active 64-bit payload from the two provided by the DMA read FIFO.
    payload_active <= payload_data(63 downto 0) when payload_select = '0' else payload_data(AXI_WIDTH-1 downto AXI_WIDTH-64);
    
  	-- Increment the sample counter with each new valid output unless the threshold has been reached, then just load the intial value.
	count_enable <= (stop_on_count and enable) or threshold;
       
	-- Use a DSP48 block to implement the sample counter for performance.
	SAMPLE_CNT : entity work.counter_32
	port map (
		clk		=> tx_clk,
		sclr	=> '0',
		ce		=> count_enable,
		load	=> threshold,
		l		=> (0=>'1', others=>'0'),
		q		=> sample_counter
	);

	-- Map the threshold flag to an output port.
	count_done <= threshold;

end structural;
